@@ -6,17 +6,21 @@ if (TestData.testData !== undefined) {
     load('jstests/audit/_audit_helpers.js');
 }
 
+var testDBName = 'audit_log_application_message';
+
 auditTest(
     'logApplicationMessage',
     function(m) {
         var msg = "it's a trap!"
+        const beforeCmd = Date.now();
         assert.commandWorked(m.getDB('admin').runCommand({ logApplicationMessage: msg }));
 
-        auditColl = getAuditEventsCollection(m);
+        const beforeLoad = Date.now();
+        auditColl = getAuditEventsCollection(m, testDBName);
         assert.eq(1, auditColl.count({
             atype: "applicationMessage",
-            ts: withinTheLastFewSeconds(),
-            'params.msg': msg,
+            ts: withinInterval(beforeCmd, beforeLoad),
+            'param.msg': msg,
             result: 0,
         }), "FAILED, audit log: " + tojson(auditColl.find().toArray()));
     },
