@@ -53,6 +53,7 @@
 #include "mongo/db/free_mon/free_mon_mongod.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/kill_sessions_local.h"
+#include "mongo/db/logical_clock.h"
 #include "mongo/db/logical_time_metadata_hook.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/op_observer.h"
@@ -67,6 +68,7 @@
 #include "mongo/db/repl/oplog_buffer_blocking_queue.h"
 #include "mongo/db/repl/repl_settings.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/replication_metrics.h"
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/repl/storage_interface.h"
 #include "mongo/db/repl/sync_tail.h"
@@ -155,7 +157,7 @@ auto makeThreadPool(const std::string& poolName) {
     threadPoolOptions.poolName = poolName;
     threadPoolOptions.onCreateThread = [](const std::string& threadName) {
         Client::initThread(threadName.c_str());
-        AuthorizationSession::get(cc())->grantInternalAuthorization();
+        AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
     };
     return stdx::make_unique<ThreadPool>(threadPoolOptions);
 }
@@ -501,6 +503,12 @@ OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationC
         });
     }
     const auto opTimeToReturn = fassert(28665, loadLastOpTime(opCtx));
+    const auto newTermStartDate = fassert(31302, loadLastWallTime(opCtx));
+
+    ReplicationMetrics::get(opCtx).setCandidateNewTermStartDate(newTermStartDate);
+
+    auto replCoord = ReplicationCoordinator::get(opCtx);
+    replCoord->createWMajorityWriteAvailabilityDateWaiter(opTimeToReturn);
 
     _shardingOnTransitionToPrimaryHook(opCtx);
     _dropAllTempCollections(opCtx);
@@ -556,6 +564,37 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalConfigDocument(Operati
     }
 }
 
+Status ReplicationCoordinatorExternalStateImpl::createLocalLastVoteCollection(
+    OperationContext* opCtx) {
+    auto status = _storageInterface->createCollection(
+        opCtx, NamespaceString(lastVoteCollectionName), CollectionOptions());
+    if (!status.isOK() && status.code() != ErrorCodes::NamespaceExists) {
+        return {ErrorCodes::CannotCreateCollection,
+                str::stream() << "Failed to create local last vote collection. Ns: "
+                              << lastVoteCollectionName
+                              << " Error: "
+                              << status.toString()};
+    }
+
+    // Make sure there's always a last vote document.
+    try {
+        writeConflictRetry(
+            opCtx, "create initial replica set lastVote", lastVoteCollectionName, [opCtx] {
+                AutoGetCollection coll(opCtx, NamespaceString(lastVoteCollectionName), MODE_X);
+                BSONObj result;
+                bool exists = Helpers::getSingleton(opCtx, lastVoteCollectionName, result);
+                if (!exists) {
+                    LastVote lastVote{OpTime::kInitialTerm, -1};
+                    Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVote.toBSON());
+                }
+            });
+    } catch (const DBException& ex) {
+        return ex.toStatus();
+    }
+
+    return Status::OK();
+}
+
 StatusWith<LastVote> ReplicationCoordinatorExternalStateImpl::loadLocalLastVoteDocument(
     OperationContext* opCtx) {
     try {
@@ -585,26 +624,24 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalLastVoteDocument(
                 // don't want to have this process interrupted due to us stepping down, since we
                 // want to be able to cast our vote for a new primary right away.
                 UninterruptibleLockGuard noInterrupt(opCtx->lockState());
-                Lock::DBLock dbWriteLock(opCtx, lastVoteDatabaseName, MODE_X);
+                AutoGetCollection coll(opCtx, NamespaceString(lastVoteCollectionName), MODE_IX);
+                WriteUnitOfWork wunit(opCtx);
 
-                // If there is no last vote document, we want to store one. Otherwise, we only want
-                // to replace it if the new last vote document would have a higher term. We both
-                // check the term of the current last vote document and insert the new document
-                // under the DBLock to synchronize the two operations.
+                // We only want to replace the last vote document if the new last vote document
+                // would have a higher term. We check the term of the current last vote document and
+                // insert the new document in a WriteUnitOfWork to synchronize the two operations.
+                // We have already ensured at startup time that there is an old document.
                 BSONObj result;
                 bool exists = Helpers::getSingleton(opCtx, lastVoteCollectionName, result);
-                if (!exists) {
-                    Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVoteObj);
-                } else {
-                    StatusWith<LastVote> oldLastVoteDoc = LastVote::readFromLastVote(result);
-                    if (!oldLastVoteDoc.isOK()) {
-                        return oldLastVoteDoc.getStatus();
-                    }
-                    if (lastVote.getTerm() > oldLastVoteDoc.getValue().getTerm()) {
-                        Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVoteObj);
-                    }
+                fassert(51241, exists);
+                StatusWith<LastVote> oldLastVoteDoc = LastVote::readFromLastVote(result);
+                if (!oldLastVoteDoc.isOK()) {
+                    return oldLastVoteDoc.getStatus();
                 }
-
+                if (lastVote.getTerm() > oldLastVoteDoc.getValue().getTerm()) {
+                    Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVoteObj);
+                }
+                wunit.commit();
                 return Status::OK();
             });
 
@@ -625,54 +662,81 @@ void ReplicationCoordinatorExternalStateImpl::setGlobalTimestamp(ServiceContext*
     setNewTimestamp(ctx, newTime);
 }
 
+Timestamp ReplicationCoordinatorExternalStateImpl::getGlobalTimestamp(ServiceContext* service) {
+    return LogicalClock::get(service)->getClusterTime().asTimestamp();
+}
+
 bool ReplicationCoordinatorExternalStateImpl::oplogExists(OperationContext* opCtx) {
     AutoGetCollection oplog(opCtx, NamespaceString::kRsOplogNamespace, MODE_IS);
     return oplog.getCollection() != nullptr;
 }
 
-StatusWith<OpTime> ReplicationCoordinatorExternalStateImpl::loadLastOpTime(
-    OperationContext* opCtx) {
-    // TODO: handle WriteConflictExceptions below
+Status ReplicationCoordinatorExternalStateImpl::_loadLastOplogEntry(OperationContext* opCtx,
+                                                                    BSONObj* result) {
     try {
         // If we are doing an initial sync do not read from the oplog.
         if (_replicationProcess->getConsistencyMarkers()->getInitialSyncFlag(opCtx)) {
             return {ErrorCodes::InitialSyncFailure, "In the middle of an initial sync."};
         }
 
-        BSONObj oplogEntry;
-
         if (!writeConflictRetry(
-                opCtx, "Load last opTime", NamespaceString::kRsOplogNamespace.ns().c_str(), [&] {
+                opCtx,
+                "Load last oplog entry",
+                NamespaceString::kRsOplogNamespace.ns().c_str(),
+                [&] {
                     return Helpers::getLast(
-                        opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), oplogEntry);
+                        opCtx, NamespaceString::kRsOplogNamespace.ns().c_str(), *result);
                 })) {
-            return StatusWith<OpTime>(ErrorCodes::NoMatchingDocument,
-                                      str::stream() << "Did not find any entries in "
-                                                    << NamespaceString::kRsOplogNamespace.ns());
+            return {ErrorCodes::NoMatchingDocument,
+                    str::stream() << "Did not find any entries in "
+                                  << NamespaceString::kRsOplogNamespace.ns()};
         }
-        BSONElement tsElement = oplogEntry[tsFieldName];
-        if (tsElement.eoo()) {
-            return StatusWith<OpTime>(ErrorCodes::NoSuchKey,
-                                      str::stream() << "Most recent entry in "
-                                                    << NamespaceString::kRsOplogNamespace.ns()
-                                                    << " missing \""
-                                                    << tsFieldName
-                                                    << "\" field");
-        }
-        if (tsElement.type() != bsonTimestamp) {
-            return StatusWith<OpTime>(ErrorCodes::TypeMismatch,
-                                      str::stream() << "Expected type of \"" << tsFieldName
-                                                    << "\" in most recent "
-                                                    << NamespaceString::kRsOplogNamespace.ns()
-                                                    << " entry to have type Timestamp, but found "
-                                                    << typeName(tsElement.type()));
-        }
-        return OpTime::parseFromOplogEntry(oplogEntry);
+
+        return Status::OK();
     } catch (const DBException& ex) {
-        return StatusWith<OpTime>(ex.toStatus());
+        return ex.toStatus();
     }
 }
 
+StatusWith<OpTime> ReplicationCoordinatorExternalStateImpl::loadLastOpTime(
+    OperationContext* opCtx) {
+    BSONObj oplogEntry;
+    auto status = _loadLastOplogEntry(opCtx, &oplogEntry);
+    if (!status.isOK()) {
+        return StatusWith<OpTime>(status);
+    }
+
+    BSONElement tsElement = oplogEntry[tsFieldName];
+    if (tsElement.eoo()) {
+        return StatusWith<OpTime>(ErrorCodes::NoSuchKey,
+                                  str::stream() << "Most recent entry in "
+                                                << NamespaceString::kRsOplogNamespace.ns()
+                                                << " missing \""
+                                                << tsFieldName
+                                                << "\" field");
+    }
+    if (tsElement.type() != bsonTimestamp) {
+        return StatusWith<OpTime>(ErrorCodes::TypeMismatch,
+                                  str::stream() << "Expected type of \"" << tsFieldName
+                                                << "\" in most recent "
+                                                << NamespaceString::kRsOplogNamespace.ns()
+                                                << " entry to have type Timestamp, but found "
+                                                << typeName(tsElement.type()));
+    }
+    return OpTime::parseFromOplogEntry(oplogEntry);
+}
+
+StatusWith<Date_t> ReplicationCoordinatorExternalStateImpl::loadLastWallTime(
+    OperationContext* opCtx) {
+    BSONObj oplogEntry;
+    auto status = _loadLastOplogEntry(opCtx, &oplogEntry);
+    if (!status.isOK()) {
+        return StatusWith<Date_t>(status);
+    }
+
+    return OpTime::parseWallTimeFromOplogEntry(oplogEntry);
+}
+
 bool ReplicationCoordinatorExternalStateImpl::isSelf(const HostAndPort& host, ServiceContext* ctx) {
     return repl::isSelf(host, ctx);
 }
