@@ -69,6 +69,7 @@
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/repl_set_config.h"
 #include "mongo/db/repl/replication_coordinator.h"
+#include "mongo/db/repl/replication_metrics.h"
 #include "mongo/db/repl/session_update_tracker.h"
 #include "mongo/db/server_parameters.h"
 #include "mongo/db/service_context.h"
@@ -91,6 +92,7 @@ using std::endl;
 namespace repl {
 
 AtomicInt32 SyncTail::replBatchLimitOperations{5 * 1000};
+AtomicInt32 SyncTail::replBatchLimitBytes{100 * 1024 * 1024};
 
 namespace {
 
@@ -139,6 +141,25 @@ public:
     }
 } exportedBatchLimitOperationsParam;
 
+class ExportedBatchLimitBytesParameter
+    : public ExportedServerParameter<int, ServerParameterType::kStartupAndRuntime> {
+public:
+    ExportedBatchLimitBytesParameter()
+        : ExportedServerParameter<int, ServerParameterType::kStartupAndRuntime>(
+              ServerParameterSet::getGlobal(),
+              "replBatchLimitBytes",
+              &SyncTail::replBatchLimitBytes) {}
+
+    virtual Status validate(const int& potentialNewValue) {
+        if (potentialNewValue < (16 * 1024 * 1024) || potentialNewValue > (100 * 1024 * 1024)) {
+            return Status(ErrorCodes::BadValue,
+                          "replBatchLimitBytes must be between 16MB and 100MB, inclusive");
+        }
+
+        return Status::OK();
+    }
+} exportedBatchLimitBytesParam;
+
 // The oplog entries applied
 Counter64 opsAppliedStats;
 ServerStatusMetricField<Counter64> displayOpsApplied("repl.apply.ops", &opsAppliedStats);
@@ -324,7 +345,7 @@ std::size_t SyncTail::calculateBatchLimitBytes(OperationContext* opCtx,
     auto oplogMaxSizeResult =
         storageInterface->getOplogMaxSize(opCtx, NamespaceString::kRsOplogNamespace);
     auto oplogMaxSize = fassert(40301, oplogMaxSizeResult);
-    return std::min(oplogMaxSize / 10, std::size_t(replBatchLimitBytes));
+    return std::min(oplogMaxSize / 10, std::size_t(replBatchLimitBytes.load()));
 }
 
 std::unique_ptr<ThreadPool> SyncTail::makeWriterPool() {
@@ -340,7 +361,7 @@ std::unique_ptr<ThreadPool> SyncTail::makeWriterPool(int threadCount) {
         // Only do this once per thread
         if (!Client::getCurrent()) {
             Client::initThreadIfNotAlready();
-            AuthorizationSession::get(cc())->grantInternalAuthorization();
+            AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
         }
     };
     auto pool = stdx::make_unique<ThreadPool>(options);
@@ -400,6 +421,17 @@ Status SyncTail::syncApply(OperationContext* opCtx,
     if (opType == OpTypeEnum::kNoop) {
         if (nss.db() == "") {
             incrementOpsAppliedStats();
+
+            auto oplogEntry = OplogEntryBase::parse(IDLParserErrorContext("syncApply"), op);
+            auto opObj = oplogEntry.getObject();
+            if (opObj.hasField(ReplicationCoordinator::newPrimaryMsgField) &&
+                opObj.getField(ReplicationCoordinator::newPrimaryMsgField).str() ==
+                    ReplicationCoordinator::newPrimaryMsg) {
+
+                invariant(oplogEntry.getWallClockTime());
+                ReplicationMetrics::get(opCtx).setParticipantNewTermDates(
+                    oplogEntry.getWallClockTime().get(), applyStartTime);
+            }
             return Status::OK();
         }
         Lock::DBLock dbLock(opCtx, nss.db(), MODE_X);
@@ -840,14 +872,14 @@ private:
         Client::initThread("ReplBatcher");
 
         BatchLimits batchLimits;
-        batchLimits.bytes =
-            calculateBatchLimitBytes(cc().makeOperationContext().get(), _storageInterface);
 
         while (true) {
             batchLimits.slaveDelayLatestTimestamp = _calculateSlaveDelayLatestTimestamp();
 
-            // Check this once per batch since users can change it at runtime.
+            // Check the limits once per batch since users can change them at runtime.
             batchLimits.ops = replBatchLimitOperations.load();
+            batchLimits.bytes =
+                calculateBatchLimitBytes(cc().makeOperationContext().get(), _storageInterface);
 
             OpQueue ops(batchLimits.ops);
             // tryPopAndWaitForMore adds to ops and returns true when we need to end a batch early.
@@ -937,15 +969,7 @@ void SyncTail::_oplogApplication(OplogBuffer* oplogBuffer,
         if (MONGO_FAIL_POINT(rsSyncApplyStop)) {
             log() << "sync tail - rsSyncApplyStop fail point enabled. Blocking until fail point is "
                      "disabled.";
-            while (MONGO_FAIL_POINT(rsSyncApplyStop)) {
-                // Tests should not trigger clean shutdown while that failpoint is active. If we
-                // think we need this, we need to think hard about what the behavior should be.
-                if (inShutdown()) {
-                    severe() << "Turn off rsSyncApplyStop before attempting clean shutdown";
-                    fassertFailedNoTrace(40304);
-                }
-                sleepmillis(10);
-            }
+            MONGO_FAIL_POINT_PAUSE_WHILE_SET(rsSyncApplyStop);
         }
 
         // Get the current value of 'minValid'.
@@ -1004,16 +1028,13 @@ void SyncTail::_oplogApplication(OplogBuffer* oplogBuffer,
             minValid = lastOpTimeInBatch;
         }
 
-        // Update various things that care about our last applied optime. Tests rely on 2 happening
-        // before 3 even though it isn't strictly necessary. The order of 1 doesn't matter.
+        // Update various things that care about our last applied optime. Tests rely on 1 happening
+        // before 2 even though it isn't strictly necessary.
 
-        // 1. Update the global timestamp.
-        setNewTimestamp(opCtx.getServiceContext(), lastOpTimeInBatch.getTimestamp());
-
-        // 2. Persist our "applied through" optime to disk.
+        // 1. Persist our "applied through" optime to disk.
         _consistencyMarkers->setAppliedThrough(&opCtx, lastOpTimeInBatch);
 
-        // 3. Ensure that the last applied op time hasn't changed since the start of this batch.
+        // 2. Ensure that the last applied op time hasn't changed since the start of this batch.
         const auto lastAppliedOpTimeAtEndOfBatch = replCoord->getMyLastAppliedOpTime();
         invariant(lastAppliedOpTimeAtStartOfBatch == lastAppliedOpTimeAtEndOfBatch,
                   str::stream() << "the last known applied OpTime has changed from "
@@ -1022,13 +1043,14 @@ void SyncTail::_oplogApplication(OplogBuffer* oplogBuffer,
                                 << lastAppliedOpTimeAtEndOfBatch.toString()
                                 << " in the middle of batch application");
 
-        // 4. Update oplog visibility by notifying the storage engine of the new oplog entries.
+        // 3. Update oplog visibility by notifying the storage engine of the new oplog entries.
         const bool orderedCommit = true;
         _storageInterface->oplogDiskLocRegister(
             &opCtx, lastOpTimeInBatch.getTimestamp(), orderedCommit);
 
-        // 5. Finalize this batch. We are at a consistent optime if our current optime is >= the
-        // current 'minValid' optime.
+        // 4. Finalize this batch. We are at a consistent optime if our current optime is >= the
+        // current 'minValid' optime. Note that recording the lastOpTime in the finalizer includes
+        // advancing the global timestamp to at least its timestamp.
         auto consistency = (lastOpTimeInBatch >= minValid)
             ? ReplicationCoordinator::DataConsistency::Consistent
             : ReplicationCoordinator::DataConsistency::Inconsistent;
@@ -1148,6 +1170,12 @@ void SyncTail::_consume(OperationContext* opCtx, OplogBuffer* oplogBuffer) {
 }
 
 void SyncTail::shutdown() {
+    // Shutdown will hang if this failpoint is enabled.
+    if (MONGO_FAIL_POINT(rsSyncApplyStop)) {
+        severe() << "Turn off rsSyncApplyStop before attempting clean shutdown";
+        fassertFailedNoTrace(40304);
+    }
+
     stdx::lock_guard<stdx::mutex> lock(_mutex);
     _inShutdown = true;
 }
@@ -1581,6 +1609,10 @@ StatusWith<OpTime> SyncTail::multiApply(OperationContext* opCtx, MultiApplier::O
         }
     }
 
+    // Increment the counter for the number of ops applied during catchup if the node is in catchup
+    // mode.
+    replCoord->incrementNumCatchUpOpsIfCatchingUp(ops.size());
+
     // We have now written all database writes and updated the oplog to match.
     return ops.back().getOpTime();
 }
