@@ -224,31 +224,6 @@ auto parseFromOplogEntryArray(const BSONObj& obj, int elem) {
     return OpTime(tsArray.Array()[elem].timestamp(), termArray.Array()[elem].Long());
 };
 
-TEST_F(SyncTailTest, SyncApplyNoNamespaceBadOp) {
-    const BSONObj op = BSON("op"
-                            << "x");
-    ASSERT_THROWS(
-        SyncTail::syncApply(_opCtx.get(), op, OplogApplication::Mode::kInitialSync).ignore(),
-        ExceptionFor<ErrorCodes::BadValue>);
-}
-
-TEST_F(SyncTailTest, SyncApplyNoNamespaceNoOp) {
-    ASSERT_OK(SyncTail::syncApply(_opCtx.get(),
-                                  BSON("op"
-                                       << "n"),
-                                  OplogApplication::Mode::kInitialSync));
-}
-
-TEST_F(SyncTailTest, SyncApplyBadOp) {
-    const BSONObj op = BSON("op"
-                            << "x"
-                            << "ns"
-                            << "test.t");
-    ASSERT_THROWS(
-        SyncTail::syncApply(_opCtx.get(), op, OplogApplication::Mode::kInitialSync).ignore(),
-        ExceptionFor<ErrorCodes::BadValue>);
-}
-
 TEST_F(SyncTailTest, SyncApplyInsertDocumentDatabaseMissing) {
     NamespaceString nss("test.t");
     auto op = makeOplogEntry(OpTypeEnum::kInsert, nss, {});
@@ -1704,6 +1679,34 @@ public:
                                 boost::none);   // post-image optime
     }
 
+    /**
+     * Creates an OplogEntry with given parameters and preset defaults for this test suite.
+     */
+    repl::OplogEntry makeOplogEntryForMigrate(const NamespaceString& ns,
+                                              repl::OpTime opTime,
+                                              repl::OpTypeEnum opType,
+                                              BSONObj object,
+                                              boost::optional<BSONObj> object2,
+                                              const OperationSessionInfo& sessionInfo,
+                                              Date_t wallClockTime) {
+        return repl::OplogEntry(opTime,         // optime
+                                1LL,            // hash
+                                opType,         // opType
+                                ns,             // namespace
+                                boost::none,    // uuid
+                                true,           // fromMigrate
+                                0,              // version
+                                object,         // o
+                                object2,        // o2
+                                sessionInfo,    // sessionInfo
+                                boost::none,    // false
+                                wallClockTime,  // wall clock time
+                                boost::none,    // statement id
+                                boost::none,    // optime of previous write within same transaction
+                                boost::none,    // pre-image optime
+                                boost::none);   // post-image optime
+    }
+
     void checkTxnTable(const OperationSessionInfo& sessionInfo,
                        const repl::OpTime& expectedOpTime,
                        Date_t expectedWallClock) {
@@ -1954,6 +1957,91 @@ TEST_F(SyncTailTxnTableTest, MultiApplyUpdatesTheTransactionTable) {
     ASSERT_TRUE(resultNoTxn.isEmpty());
 }
 
+TEST_F(SyncTailTxnTableTest, SessionMigrationNoOpEntriesShouldUpdateTxnTable) {
+    const auto insertLsid = makeLogicalSessionIdForTest();
+    OperationSessionInfo insertSessionInfo;
+    insertSessionInfo.setSessionId(insertLsid);
+    insertSessionInfo.setTxnNumber(3);
+    auto date = Date_t::now();
+
+    auto innerOplog = makeOplogEntry(nss(),
+                                     {Timestamp(10, 10), 1},
+                                     repl::OpTypeEnum::kInsert,
+                                     BSON("_id" << 1),
+                                     boost::none,
+                                     insertSessionInfo,
+                                     date);
+
+    auto outerInsertDate = Date_t::now();
+    auto insertOplog = makeOplogEntryForMigrate(nss(),
+                                                {Timestamp(40, 0), 1},
+                                                repl::OpTypeEnum::kNoop,
+                                                BSON("$sessionMigrateInfo" << 1),
+                                                innerOplog.toBSON(),
+                                                insertSessionInfo,
+                                                outerInsertDate);
+
+    auto writerPool = SyncTail::makeWriterPool();
+    SyncTail syncTail(
+        nullptr, getConsistencyMarkers(), getStorageInterface(), multiSyncApply, writerPool.get());
+    ASSERT_OK(syncTail.multiApply(_opCtx.get(), {insertOplog}));
+
+    checkTxnTable(insertSessionInfo, {Timestamp(40, 0), 1}, outerInsertDate);
+}
+
+TEST_F(SyncTailTxnTableTest, PreImageNoOpEntriesShouldNotUpdateTxnTable) {
+    const auto preImageLsid = makeLogicalSessionIdForTest();
+    OperationSessionInfo preImageSessionInfo;
+    preImageSessionInfo.setSessionId(preImageLsid);
+    preImageSessionInfo.setTxnNumber(3);
+    auto preImageDate = Date_t::now();
+
+    auto preImageOplog = makeOplogEntryForMigrate(nss(),
+                                                  {Timestamp(30, 0), 1},
+                                                  repl::OpTypeEnum::kNoop,
+                                                  BSON("_id" << 1),
+                                                  boost::none,
+                                                  preImageSessionInfo,
+                                                  preImageDate);
+
+    auto writerPool = SyncTail::makeWriterPool();
+    SyncTail syncTail(
+        nullptr, getConsistencyMarkers(), getStorageInterface(), multiSyncApply, writerPool.get());
+    ASSERT_OK(syncTail.multiApply(_opCtx.get(), {preImageOplog}));
+
+    DBDirectClient client(_opCtx.get());
+    auto result = client.findOne(NamespaceString::kSessionTransactionsTableNamespace.ns(),
+                                 {BSON(SessionTxnRecord::kSessionIdFieldName
+                                       << preImageSessionInfo.getSessionId()->toBSON())});
+    ASSERT_TRUE(result.isEmpty());
+}
+
+TEST_F(SyncTailTxnTableTest, NonMigrateNoOpEntriesShouldNotUpdateTxnTable) {
+    const auto lsid = makeLogicalSessionIdForTest();
+    OperationSessionInfo sessionInfo;
+    sessionInfo.setSessionId(lsid);
+    sessionInfo.setTxnNumber(3);
+
+    auto oplog = makeOplogEntry(nss(),
+                                {Timestamp(30, 0), 1},
+                                repl::OpTypeEnum::kNoop,
+                                BSON("_id" << 1),
+                                boost::none,
+                                sessionInfo,
+                                Date_t::now());
+
+    auto writerPool = SyncTail::makeWriterPool();
+    SyncTail syncTail(
+        nullptr, getConsistencyMarkers(), getStorageInterface(), multiSyncApply, writerPool.get());
+    ASSERT_OK(syncTail.multiApply(_opCtx.get(), {oplog}));
+
+    DBDirectClient client(_opCtx.get());
+    auto result = client.findOne(
+        NamespaceString::kSessionTransactionsTableNamespace.ns(),
+        {BSON(SessionTxnRecord::kSessionIdFieldName << sessionInfo.getSessionId()->toBSON())});
+    ASSERT_TRUE(result.isEmpty());
+}
+
 TEST_F(IdempotencyTest, EmptyCappedNamespaceNotFound) {
     // Create a BSON "emptycapped" command.
     auto emptyCappedCmd = BSON("emptycapped" << nss.coll());
