@@ -374,16 +374,22 @@ Status MigrationDestinationManager::start(OperationContext* opCtx,
     return Status::OK();
 }
 
-void MigrationDestinationManager::cloneDocumentsFromDonor(
+repl::OpTime MigrationDestinationManager::cloneDocumentsFromDonor(
     OperationContext* opCtx,
     stdx::function<void(OperationContext*, BSONObj)> insertBatchFn,
     stdx::function<BSONObj(OperationContext*)> fetchBatchFn) {
 
     ProducerConsumerQueue<BSONObj> batches(1);
+    repl::OpTime lastOpApplied;
+
     stdx::thread inserterThread{[&] {
         Client::initThreadIfNotAlready("chunkInserter");
         auto inserterOpCtx = Client::getCurrent()->makeOperationContext();
-        auto consumerGuard = MakeGuard([&] { batches.closeConsumerEnd(); });
+        auto consumerGuard = MakeGuard([&] {
+            batches.closeConsumerEnd();
+            lastOpApplied = repl::ReplClientInfo::forClient(inserterOpCtx->getClient()).getLastOp();
+        });
+
         try {
             while (true) {
                 auto nextBatch = batches.pop(inserterOpCtx.get());
@@ -419,6 +425,8 @@ void MigrationDestinationManager::cloneDocumentsFromDonor(
             break;
         }
     }
+
+    return lastOpApplied;
 }
 
 Status MigrationDestinationManager::abort(const MigrationSessionId& sessionId) {
@@ -635,36 +643,45 @@ void MigrationDestinationManager::cloneCollectionIndexesAndOptions(OperationCont
             collection = db->getCollection(opCtx, nss);
         }
 
-        MultiIndexBlock indexer(opCtx, collection);
-        indexer.removeExistingIndexes(&donorIndexSpecs);
 
-        if (!donorIndexSpecs.empty()) {
-            // Only copy indexes if the collection does not have any documents.
-            uassert(ErrorCodes::CannotCreateCollection,
-                    str::stream() << "aborting, shard is missing " << donorIndexSpecs.size()
-                                  << " indexes and "
-                                  << "collection is not empty. Non-trivial "
-                                  << "index creation should be scheduled manually",
-                    collection->numRecords(opCtx) == 0);
-
-            auto indexInfoObjs = indexer.init(donorIndexSpecs);
-            uassert(ErrorCodes::CannotCreateIndex,
-                    str::stream() << "failed to create index before migrating data. "
-                                  << " error: "
-                                  << redact(indexInfoObjs.getStatus()),
-                    indexInfoObjs.isOK());
-
-            WriteUnitOfWork wunit(opCtx);
-            indexer.commit();
-
-            for (auto&& infoObj : indexInfoObjs.getValue()) {
-                // make sure to create index on secondaries as well
-                serviceContext->getOpObserver()->onCreateIndex(
-                    opCtx, collection->ns(), collection->uuid(), infoObj, true /* fromMigrate */);
-            }
-
-            wunit.commit();
+        auto indexCatalog = collection->getIndexCatalog();
+        {
+            // Use MultiIndexBlock to filter existing indexes but not to build indexes.
+            MultiIndexBlock indexer(opCtx, collection);
+            indexer.removeExistingIndexes(&donorIndexSpecs);
+            indexer.abortWithoutCleanup();
         }
+        const auto& indexSpecs = donorIndexSpecs;
+
+        if (indexSpecs.empty()) {
+            return;
+        }
+
+        // Only copy indexes if the collection does not have any documents.
+        uassert(ErrorCodes::CannotCreateCollection,
+                str::stream() << "aborting, shard is missing " << indexSpecs.size()
+                              << " indexes and "
+                              << "collection is not empty. Non-trivial "
+                              << "index creation should be scheduled manually",
+                collection->numRecords(opCtx) == 0);
+
+        WriteUnitOfWork wunit(opCtx);
+
+        for (const auto& spec : indexSpecs) {
+            // Make sure to create index on secondaries as well. Oplog entry must be written before
+            // the index is added to the index catalog for correct rollback operation.
+            // See SERVER-35780 and SERVER-35070.
+            serviceContext->getOpObserver()->onCreateIndex(
+                opCtx, collection->ns(), *(collection->uuid()), spec, true /* fromMigrate */);
+
+            // Since the collection is empty, we can add and commit the index catalog entry within
+            // a single WUOW.
+            uassertStatusOKWithContext(
+                indexCatalog->createIndexOnEmptyCollection(opCtx, spec),
+                str::stream() << "failed to create index before migrating data: " << redact(spec));
+        }
+
+        wunit.commit();
     }
 }
 
@@ -674,7 +691,7 @@ void MigrationDestinationManager::_migrateThread() {
 
 
     if (AuthorizationManager::get(opCtx->getServiceContext())->isAuthEnabled()) {
-        AuthorizationSession::get(opCtx->getClient())->grantInternalAuthorization();
+        AuthorizationSession::get(opCtx->getClient())->grantInternalAuthorization(opCtx.get());
     }
 
     try {
@@ -774,6 +791,7 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx) {
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep2);
     }
 
+    repl::OpTime lastOpApplied;
     {
         // 3. Initial bulk clone
         setState(CLONE);
@@ -860,7 +878,9 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx) {
             return res.response;
         };
 
-        cloneDocumentsFromDonor(opCtx, insertBatchFn, fetchBatchFn);
+        // If running on a replicated system, we'll need to flush the docs we cloned to the
+        // secondaries
+        lastOpApplied = cloneDocumentsFromDonor(opCtx, insertBatchFn, fetchBatchFn);
 
         timing.done(3);
         MONGO_FAIL_POINT_PAUSE_WHILE_SET(migrateThreadHangAtStep3);
@@ -872,10 +892,6 @@ void MigrationDestinationManager::_migrateDriver(OperationContext* opCtx) {
         }
     }
 
-    // If running on a replicated system, we'll need to flush the docs we cloned to the
-    // secondaries
-    repl::OpTime lastOpApplied = repl::ReplClientInfo::forClient(opCtx->getClient()).getLastOp();
-
     const BSONObj xferModsRequest = createTransferModsRequest(_nss, *_sessionId);
 
     {
