@@ -141,6 +141,10 @@ bool shouldActivateFailCommandFailPoint(const BSONObj& data, StringData cmdName,
         }
     }
 
+    if (!client->session()) {
+        return false;
+    }
+
     for (auto&& failCommand : data.getObjectField("failCommands")) {
         if (failCommand.type() == String && failCommand.valueStringData() == cmdName) {
             return true;
@@ -701,6 +705,10 @@ void execCommandDatabase(OperationContext* opCtx,
             str::stream() << "Invalid database name: '" << dbname << "'",
             NamespaceString::validDBName(dbname, NamespaceString::DollarInDbNameBehavior::Allow));
 
+        if (sessionOptions.getAutocommit()) {
+            uassertStatusOK(CommandHelpers::canUseTransactions(dbname, command->getName()));
+        }
+
         // Session ids are forwarded in requests, so commands that require roundtrips between
         // servers may result in a deadlock when a server tries to check out a session it is already
         // using to service an earlier operation in the command's chain. To avoid this, only check
@@ -881,7 +889,7 @@ void execCommandDatabase(OperationContext* opCtx,
                 uassertStatusOK(shardingState->canAcceptShardedCommands());
             }
 
-            behaviors.advanceConfigOptimeFromRequestMetadata(opCtx);
+            behaviors.advanceConfigOpTimeFromRequestMetadata(opCtx);
         }
 
         oss.setAllowImplicitCollectionCreation(allowImplicitCollectionCreationField);
