@@ -73,7 +73,8 @@ public:
                        const std::string& path,
                        ClockSource* cs,
                        const std::string& extraOpenOptions,
-                       size_t cacheSizeGB,
+                       size_t cacheSizeMB,
+                       size_t maxCacheOverflowFileSizeMB,
                        bool durable,
                        bool ephemeral,
                        bool repair,
@@ -167,6 +168,8 @@ public:
     virtual void endBackup(OperationContext* opCtx);
 
     virtual Status hotBackup(OperationContext* opCtx, const std::string& path) override;
+    virtual Status hotBackupTar(OperationContext* opCtx, const std::string& path) override;
+    virtual Status hotBackup(OperationContext* opCtx, const percona::S3BackupParameters& s3params) override;
 
     virtual int64_t getIdentSize(OperationContext* opCtx, StringData ident) override;
 
@@ -315,6 +318,13 @@ private:
     class WiredTigerJournalFlusher;
     class WiredTigerCheckpointThread;
 
+    // srcPath, destPath, session, cursor
+    typedef std::tuple<boost::filesystem::path, boost::filesystem::path, std::shared_ptr<WiredTigerSession>, WT_CURSOR*> DBTuple;
+    // srcPath, destPath, filename, size to copy
+    typedef std::tuple<boost::filesystem::path, boost::filesystem::path, boost::uintmax_t, std::time_t> FileTuple;
+
+    Status _hotBackupPopulateLists(OperationContext* opCtx, const std::string& path, std::vector<DBTuple>& dbList, std::vector<FileTuple>& filesList);
+
     /**
      * Opens a connection on the WiredTiger database 'path' with the configuration 'wtOpenConfig'.
      * Only returns when successful. Intializes both '_conn' and '_fileVersion'.
