@@ -37,6 +37,8 @@
 #include "mongo/base/checked_cast.h"
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/concurrency/d_concurrency.h"
+#include "mongo/db/db_raii.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_record_store.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_recovery_unit.h"
@@ -45,6 +47,7 @@
 #include "mongo/util/assert_util.h"
 #include "mongo/util/scopeguard.h"
 
+
 namespace mongo {
 
 using std::string;
@@ -83,4 +86,26 @@ BSONObj WiredTigerServerStatusSection::generateSection(OperationContext* opCtx,
     return bob.obj();
 }
 
+OplogStonesServerStatusSection::OplogStonesServerStatusSection()
+    : ServerStatusSection("oplogTruncation") {}
+
+bool OplogStonesServerStatusSection::includeByDefault() const {
+    return true;
+}
+
+BSONObj OplogStonesServerStatusSection::generateSection(OperationContext* opCtx,
+                                                        const BSONElement& configElement) const {
+    BSONObjBuilder builder;
+    {
+        AutoGetCollectionForReadCommand ctx(opCtx, NamespaceString::kRsOplogNamespace);
+        Collection* oplogColl = ctx.getCollection();
+        if (oplogColl) {
+            auto oplogRS = checked_cast<WiredTigerRecordStore*>(oplogColl->getRecordStore());
+            oplogRS->getOplogTruncateStats(builder);
+        }
+    }
+    return builder.obj();
+}
+
+
 }  // namespace mongo
